#!/usr/bin/env python
import sys
import os
import re
import json
import logging
import rich.console
import time

import relecov_tools.utils
from relecov_tools.config_json import ConfigJson
from relecov_tools.rest_api import RestApi

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class FeedDatabase:
    def __init__(
        self,
        user=None,
        passwd=None,
        json_file=None,
        type_of_info=None,
        database_server=None,
    ):
        if user is None:
            user = relecov_tools.utils.prompt_text(
                msg="Enter username for upload data to server"
            )
        self.user = user
        if passwd is None:
            passwd = relecov_tools.utils.prompt_text(msg="Enter credential password")
        self.passwd = passwd
        self.config_json = ConfigJson()
        if json_file is None:
            json_file = relecov_tools.utils.prompt_path(
                msg="Select the json file which have the data to map"
            )
        if not os.path.isfile(json_file):
            log.error("json data file %s does not exist ", json_file)
            stderr.print(f"[red] json data file {json_file} does not exist")
            sys.exit(1)
        self.json_data = relecov_tools.utils.read_json_file(json_file)
        self.json_file = json_file
        schema = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            "schema",
            self.config_json.get_topic_data("json_schemas", "relecov_schema"),
        )

        self.schema = relecov_tools.utils.read_json_file(schema)

        if type_of_info is None:
            type_of_info = relecov_tools.utils.prompt_selection(
                "Select:",
                ["sample", "bioinfodata", "variantdata"],
            )
        self.type_of_info = type_of_info

        if database_server is None:
            database_server = relecov_tools.utils.prompt_selection(
                "Select:",
                ["iskylims", "relecov"],
            )
        self.server_type = database_server
        # Get database settings
        try:
            self.database_settings = self.config_json.get_topic_data(
                "external_url", database_server
            )
        except KeyError:
            log.error("Unable to get parameters for dataserver")
            stderr.print(f"[red] Unable to fetch parameters data for {database_server}")
            sys.exit(1)
        self.database_server = self.database_settings["server"]
        self.database_url = self.database_settings["url"]

        self.database_rest_api = RestApi(self.database_server, self.database_url)

    def get_schema_ontology_values(self):
        """Read the schema and extract the values of ontology with the label"""
        ontology_dict = {}
        for prop, values in self.schema["properties"].items():
            if "ontology" in values and values["ontology"] != "":
                ontology_dict[values["ontology"]] = prop
        return ontology_dict

    def map_iskylims_sample_fields_values(self, sample_fields, s_project_fields):
        """Map the values to the properties send to databasee
        in json schema based on label
        """
        # collection_method = self.get_sample_type()
        sample_list = []
        s_fields = list(sample_fields.keys())
        for row in self.json_data:
            s_dict = {}
            for key, value in row.items():
                found_ontology = re.search(r"(.+) \[\w+:.*", value)
                if found_ontology:
                    # remove the ontology data from item value
                    value = found_ontology.group(1)
                if key in s_project_fields:
                    s_dict[key] = value
                elif key in s_fields:
                    s_dict[sample_fields[key]] = value
                else:
                    # just for debug loginng write the fields that will not
                    # be inlcuded in iSkyLIMS request
                    log.info("not key %s in iSkyLIMS", key)
            # include the fix value
            fixed_value = self.config_json.get_configuration("iskylims_fixed_values")
            for prop, val in fixed_value.items():
                s_dict[prop] = val
            # Adding tha specimen_source field for setting sampleType
            s_dict["sampleType"] = row["specimen_source"]
            sample_list.append(s_dict)
        return sample_list

    def get_iskylims_fields_sample(self):
        """2 requests are sent to iSkyLIMS. One for getting the sample fields
        These fields are mapped using the ontology.
        The second request is for getting the sample project fields. These are
        mapped using the label value.
        """

        sample_fields = {}
        s_project_fields = []
        # get the ontology values for mapping values in sample fields
        ontology_dict = self.get_schema_ontology_values()
        sample_url = self.database_settings["url_sample_fields"]
        try:
            sample_fields_raw = self.database_rest_api.get_request(sample_url, "", "")
        except AttributeError:
            log.error("Unable to connect to server %s", self.database_server)
            stderr.print(f"[red] Unable to connect to server {self.database_server}")
            sys.exit(1)
        if "ERROR" in sample_fields_raw:
            log.error(
                "Unable to get parameters. Received error code %s",
                sample_fields_raw["ERROR"],
            )
            stderr.print(
                f"[red] Unable to fetch data. Received error {sample_fields_raw['ERROR']}"
            )
            sys.exit(1)

        for key, values in sample_fields_raw["DATA"].items():
            if "ontology" in values:
                try:
                    property = ontology_dict[values["ontology"]]
                    # sample_fields has a key the label in metadata and value
                    # the field name for sample
                    sample_fields[property] = values["field_name"]
                except KeyError as e:
                    stderr.print(f"[red]Error in map ontology {e}")
            else:
                # for the ones that do no have ontologuy label is the sample field
                # and the value is empty
                # sample_fields[key] = ""
                log.info("not ontology for item  %s", values["field_name"])
        # fetch label for sample Project
        s_project_url = self.database_settings["url_project_fields"]
        param = self.database_settings["param_sample_project"]
        p_name = self.database_settings["project_name"]
        s_project_fields_raw = self.database_rest_api.get_request(
            s_project_url, param, p_name
        )
        if "ERROR" in s_project_fields_raw:
            log.error(
                "Unable to get parameters. Received error code %s",
                s_project_fields_raw["ERROR"],
            )
            stderr.print(
                f"[red] Unable to fetch data. Received error {s_project_fields_raw['ERROR']}"
            )
            sys.exit(1)
        for field in s_project_fields_raw["DATA"]:
            s_project_fields.append(field["sampleProjectFieldName"])

        return [sample_fields, s_project_fields]

    def map_relecov_sample_data(self):
        """Select the values from self.json_data"""
        field_values = []
        r_fields = self.config_json.get_configuration("relecov_sample_metadata")

        for row in self.json_data:
            s_dict = {}
            for r_field in r_fields:
                if r_field in row:
                    s_dict[r_field] = row[r_field]
                else:
                    s_dict[r_field] = None
            field_values.append(s_dict)
        return field_values

    def update_database(self, field_values, post_url):
        """Send the request to update database"""
        suces_count = 0
        request_count = 0
        for chunk in field_values:
            req_sample = ""
            request_count += 1
            if "sample_name" in chunk:
                stderr.print(
                    f"[blue] sending request for sample {chunk['sample_name']}"
                )
                req_sample = chunk["sample_name"]
            elif "sequencing_sample_id" in chunk:
                stderr.print(
                    f"[blue] sending request for sample {chunk['sequencing_sample_id']}"
                )
                req_sample = chunk["sequencing_sample_id"]
            result = self.database_rest_api.post_request(
                json.dumps(chunk),
                {"user": self.user, "pass": self.passwd},
                self.database_settings[post_url],
            )

            if "ERROR" in result:
                if result["ERROR"] == "Server not available":
                    # retry to connect to server
                    for i in range(10):
                        # wait 5 sec before resending the request
                        time.sleep(5)
                        result = self.database_rest_api.post_request(
                            json.dumps(chunk),
                            {"user": self.user, "pass": self.passwd},
                            self.database_settings[post_url],
                        )
                        if "ERROR" not in result:
                            break
                    if i == 9 and "ERROR" in result:
                        log.error("Unable to sent the request to remote server")
                        stderr.print(
                            "[red] Unable to sent the request to remote server"
                        )
                        sys.exit(1)

                elif "is not defined" in result["ERROR_TEST"].lower():
                    log.warning(
                        "Request to %s for %s was not accepted",
                        self.database_server,
                        req_sample,
                    )
                    stderr.print(f"[yellow] Warning: {req_sample} is not defined")
                    continue
                elif "already defined" in result["ERROR_TEST"].lower():
                    log.warning(
                        "Request to %s for %s was not accepted",
                        self.database_server,
                        req_sample,
                    )
                    stderr.print(
                        f"[yellow] Warning request for {req_sample} already defined"
                    )
                    continue
                else:
                    log.error("Request to %s was not accepted", self.database_server)
                    stderr.print(
                        f"[red] Error {result['ERROR']} when sending request to {self.database_server}"
                    )
                    sys.exit(1)
            log.info(
                "stored data in %s iskylims for sample %s",
                self.database_server,
                req_sample,
            )
            stderr.print(f"[green] Successful request for {req_sample}")
            suces_count += 1
        if request_count == suces_count:
            stderr.print(
                f"[gren] All information was sent sucessfuly to {self.server_type}"
            )
        else:
            stderr.print(
                "[yellow] Some of your requests were not successful stored in database"
            )
            stderr.print(f"[yellow] {suces_count} of the {request_count} were done ok")
        return

    def store_data(self):
        """Collect data from json file and split them to store data in iSkyLIMS
        and in Relecov Platform
        """

        map_fields = {}  #
        if self.type_of_info == "sample":
            if self.server_type == "iskylims":
                stderr.print(f"[blue] Getting sample fields from {self.server_type}")
                sample_fields, s_project_fields = self.get_iskylims_fields_sample()
                stderr.print("[blue] Selecting sample fields")
                map_fields = self.map_iskylims_sample_fields_values(
                    sample_fields, s_project_fields
                )
            else:
                stderr.print("[blue] Selecting sample fields")
                map_fields = self.map_relecov_sample_data()
            post_url = "store_samples"

        elif self.type_of_info == "bioinfodata":
            post_url = "bioinfodata"
            map_fields = self.json_data

        elif self.type_of_info == "variantdata":
            post_url = "variantdata"
            map_fields = self.json_data
        else:
            stderr.print("[red] Invalid type to upload to database")
            sys.exit(1)

        self.update_database(map_fields, post_url)
        stderr.print(f"[green] Upload process to {self.server_type} completed")



#!/usr/bin/env python

import os
import sys
import re
import logging
import glob
import rich.console
from datetime import datetime
from yaml import YAMLError

import relecov_tools.utils
from relecov_tools.config_json import ConfigJson

# import relecov_tools.json_schema

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class BioinfoMetadata:
    def __init__(
        self,
        json_file=None,
        input_folder=None,
        output_folder=None,
    ):
        if json_file is None:
            json_file = relecov_tools.utils.prompt_path(
                msg="Select the json file that was created by the read-lab-metadata"
            )
        if not os.path.isfile(json_file):
            log.error("json file %s does not exist ", json_file)
            stderr.print(f"[red] file {json_file} does not exist")
            sys.exit(1)
        self.json_file = json_file

        if input_folder is None:
            self.input_folder = relecov_tools.utils.prompt_path(
                msg="Select the input folder"
            )
        else:
            self.input_folder = input_folder
        if output_folder is None:
            self.output_folder = relecov_tools.utils.prompt_path(
                msg="Select the output folder"
            )
        else:
            self.output_folder = output_folder

        config_json = ConfigJson()
        self.configuration = config_json
        required_files = self.configuration.get_topic_data(
            "bioinfo_analysis", "required_file"
        )
        self.req_files = {}
        for key, file in required_files.items():
            f_path = os.path.join(self.input_folder, file)
            if not os.path.isfile(f_path):
                log.error("File %s does not exist", file)
                stderr.print(f"[red]File {file} does not exist")
                sys.exit(1)
            self.req_files[key] = f_path

    def add_fixed_values(self, j_data):
        """include the fixed data defined in configuration"""
        f_values = self.configuration.get_topic_data("bioinfo_analysis", "fixed_values")
        for row in j_data:
            for field, value in f_values.items():
                row[field] = value
        return j_data

    def include_data_from_mapping_stats(self, j_data):
        """By processing mapping stats file the following information is
        included in schema properties:  depth_of_coverage_value, lineage_name,
        number_of_variants_in_consensus, number_of_variants_with_effect,
        per_genome_greater_10x. per_Ns. per_reads_host, per_reads_virus.
        per_unmapped, qc_filtered, reference_genome_accession
        """
        # position of the sample columns inside mapping file
        sample_position = 4
        map_data = relecov_tools.utils.read_csv_file_return_dict(
            self.req_files["mapping_stats"], "\t", sample_position
        )

        mapping_fields = self.configuration.get_topic_data(
            "bioinfo_analysis", "mapping_stats"
        )

        for row in j_data:
            for field, value in mapping_fields.items():
                try:
                    row[field] = map_data[row["sequencing_sample_id"]][value]
                except KeyError as e:
                    log.error("Field %s not found in mapping stats", e)
                    stderr.print(f"[red]Field {e} not found in mapping stats")
                    sys.exit(1)
        return j_data

    def include_pangolin_data(self, j_data):
        """Include pangolin data collecting form each file generated by pangolin"""
        mapping_fields = self.configuration.get_topic_data(
            "bioinfo_analysis", "mapping_pangolin"
        )
        for row in j_data:
            if "-" in row["sequencing_sample_id"]:
                sample_name = row["sequencing_sample_id"].replace("-", "_")
            else:
                sample_name = row["sequencing_sample_id"]

            f_name_regex = sample_name + ".pangolin.*.csv"
            f_path = os.path.join(self.input_folder, f_name_regex)
            pangolin_sample_file = glob.glob(f_path)
            if pangolin_sample_file:
                if len(pangolin_sample_file) == 1:
                    try:
                        result_regex = re.search(
                            "(.*)\.pangolin\.(.*)\.csv", pangolin_sample_file[0]
                        )
                        row["lineage_analysis_date"] = result_regex.group(2)
                        row["lineage_analysis_date"] = datetime.strptime(
                            row["lineage_analysis_date"], "%Y%m%d"
                        ).strftime("%Y-%m-%d")
                    except Exception as e:
                        stderr.print(
                            f"[red] Pattern not found in file name. Error: {e}"
                        )
                    try:
                        f_data = relecov_tools.utils.read_csv_file_return_dict(
                            result_regex.group(), ","
                        )
                    except FileNotFoundError as e:
                        log.error("File %s not found ", e)
                        stderr.print(f"[red]File {e} not found")
                        # When file does not exist set all values to empty
                        for field, value in mapping_fields.items():
                            row[field] = ""
                        continue
                        # sys.exit(1)
                    pang_key = list(f_data.keys())[0]
                    for field, value in mapping_fields.items():
                        row[field] = f_data[pang_key][value]
                else:
                    # We need to handle this when more than one analysis in the folder. How can we do this? Use the last one?
                    stderr.print(
                        "[red] More than one pangolin file found for the same sample "
                    )
                    sys.exit(1)
            else:
                stderr.print(f"[yellow] No pangolin file for sample: {sample_name}")

        return j_data

    def include_consensus_data(self, j_data):
        """Include genome length, name, file name, path and md5 by preprocessing
        each file of consensus.fa
        """

        mapping_fields = self.configuration.get_topic_data(
            "bioinfo_analysis", "mapping_consensus"
        )
        for row in j_data:
            if "-" in row["sequencing_sample_id"]:
                sample_name = row["sequencing_sample_id"].replace("-", "_")
            else:
                sample_name = row["sequencing_sample_id"]
            f_name = sample_name + ".consensus.fa"
            f_path = os.path.join(self.input_folder, f_name)
            try:
                record_fasta = relecov_tools.utils.read_fasta_return_SeqIO_instance(
                    f_path
                )
            except FileNotFoundError as e:
                log.error("File %s not found ", e)
                stderr.print(f"[red]File {e} not found")
                for item in mapping_fields:
                    row[item] = ""
                continue
            row["consensus_genome_length"] = str(len(record_fasta))
            row["consensus_sequence_name"] = record_fasta.description
            row["consensus_sequence_filepath"] = self.input_folder
            row["consensus_sequence_filename"] = f_name
            row["consensus_sequence_md5"] = relecov_tools.utils.calculate_md5(f_path)
            base_calculation = int(row["read_length"]) * len(record_fasta)
            if row["sequencing_sample_id"] != "":
                row["number_of_base_pairs_sequenced"] = str(base_calculation * 2)
            else:
                row["number_of_base_pairs_sequenced"] = str(base_calculation)

        return j_data

    def include_long_table_path(self, j_data):
        """Include the variant long table path by searchin the in input folder
        the file name that contains long_table.csv
        """
        condition = os.path.join(self.input_folder, "*long_table.csv")
        f_path = relecov_tools.utils.get_files_match_condition(condition)
        if len(f_path) == 0:
            long_table_path = ""
        else:
            long_table_path = f_path[0]
        for row in j_data:
            row["long_table_path"] = long_table_path
        return j_data

    def include_variant_metrics(self, j_data):
        """Include the # Ns per 100kb consensus from the summary variant
        metric file_exists
        """
        map_data = relecov_tools.utils.read_csv_file_return_dict(
            self.req_files["variants_metrics"], ","
        )
        mapping_fields = self.configuration.get_topic_data(
            "bioinfo_analysis", "mapping_variant_metrics"
        )
        for row in j_data:
            for field, value in mapping_fields.items():
                try:
                    row[field] = map_data[row["sequencing_sample_id"]][value]
                except KeyError as e:
                    log.error("Field %s not found in mapping stats", e)
                    stderr.print(f"[red]Field {e} not found in mapping stats")
                    sys.exit(1)
        return j_data

    def include_software_versions(self, j_data):
        """Include versions from the yaml version file"""
        version_fields = self.configuration.get_topic_data(
            "bioinfo_analysis", "mapping_version"
        )

        try:
            versions = relecov_tools.utils.read_yml_file(self.req_files["versions"])
        except YAMLError as e:
            log.error("Unable to process version file return error %s", e)
            stderr.print("[red]Unable to process version file")
            stderr.print(f" {e}")
            sys.exit(1)
        for row in j_data:
            for field, version_data in version_fields.items():

                for key, value in version_data.items():

                    row[field] = versions[key][value]
        return j_data

    def collect_info_from_lab_json(self):
        """Create the list of dictionaries from the data that is on json lab
        metadata file. Return j_data that is used to add the rest of the fields
        """
        try:
            json_lab_data = relecov_tools.utils.read_json_file(self.json_file)
        except ValueError:
            log.error("%s invalid json file", self.json_file)
            stderr.print(f"[red] {self.json_file} invalid json file")
            sys.exit(1)
        #        j_data = []
        #        mapping_fields = self.configuration.get_topic_data(
        #            "bioinfo_analysis", "required_fields_from_lab_json"
        #        )
        #        for row in json_lab_data:
        #            j_data_dict = {}
        #            for lab_field, bio_field in mapping_fields.items():
        #                j_data_dict[bio_field] = row[lab_field]
        #            j_data.append(j_data_dict)
        return json_lab_data

    def create_bioinfo_file(self):
        """Create the bioinfodata json with collecting information from lab
        metadata json, mapping_stats, and more information from the files
        inside input directory
        """
        stderr.print("[blue]Reading lab metadata json")
        j_data = self.collect_info_from_lab_json()
        stderr.print("[blue]Adding fixed values")
        j_data = self.add_fixed_values(j_data)
        stderr.print("[blue]Adding data from mapping stats")
        j_data = self.include_data_from_mapping_stats(j_data)
        stderr.print("[blue]Adding software versions")
        j_data = self.include_software_versions(j_data)
        stderr.print("[blue]Adding summary variant metrics")
        j_data = self.include_variant_metrics(j_data)
        stderr.print("[blue]Adding pangolin information")
        j_data = self.include_pangolin_data(j_data)
        stderr.print("[blue]Adding consensus data")
        j_data = self.include_consensus_data(j_data)
        stderr.print("[blue]Adding variant long table path")
        j_data = self.include_long_table_path(j_data)
        file_name = (
            "bioinfo_" + os.path.splitext(os.path.basename(self.json_file))[0] + ".json"
        )
        stderr.print("[blue]Writting output json file")
        os.makedirs(self.output_folder, exist_ok=True)
        file_path = os.path.join(self.output_folder, file_name)
        relecov_tools.utils.write_json_fo_file(j_data, file_path)
        stderr.print("[green]Sucessful creation of bioinfo analyis file")
        return True



#!/usr/bin/env python
from collections import OrderedDict
from datetime import datetime
import json
import jsonschema
from jsonschema import Draft202012Validator
from relecov_tools.config_json import ConfigJson
import logging
import rich.console
import os
import sys

# import jsonschema
import relecov_tools.utils


log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class MappingSchema:
    def __init__(
        self,
        relecov_schema=None,
        json_file=None,
        destination_schema=None,
        schema_file=None,
        output_folder=None,
    ):
        config_json = ConfigJson()
        if relecov_schema is None:
            relecov_schema = os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                "schema",
                config_json.get_topic_data("json_schemas", "relecov_schema"),
            )
        else:
            if os.path.isfile(relecov_schema):
                log.error("Relecov schema file %s does not exists", relecov_schema)
                stderr.print(
                    "[red] Relecov schema " + relecov_schema + "does not exists"
                )
                exit(1)
        rel_schema_json = relecov_tools.utils.read_json_file(relecov_schema)
        try:
            Draft202012Validator.check_schema(rel_schema_json)
        except jsonschema.ValidationError:
            log.error("Relecov schema does not fulfil Draft 202012 Validation ")
            stderr.print("[red] Relecov schema does not fulfil Draft 202012 Validation")
            sys.exit(1)
        self.relecov_schema = rel_schema_json

        if json_file is None:
            json_file = relecov_tools.utils.prompt_path(
                msg="Select the json file which have the data to map"
            )
        if not os.path.isfile(json_file):
            log.error("json data file %s does not exist ", json_file)
            stderr.print(f"[red] json data file {json_file} does not exist")
            sys.exit(1)
        self.json_data = relecov_tools.utils.read_json_file(json_file)
        self.json_file = json_file
        if destination_schema is None:
            self.destination_schema = relecov_tools.utils.prompt_selection(
                msg="Select ENA, GISAID for already defined schemas or other for custom",
                choices=["ENA", "GISAID", "other"],
            )
        else:
            self.destination_schema = destination_schema
        if self.destination_schema == "other":
            if schema_file is None:
                self.schema_file = relecov_tools.utils.prompt_path(
                    msg="Select the json schema file to map your data"
                )
            else:
                self.schema_file = schema_file
            if not os.path.exists(self.schema_file):
                log.error(
                    "Schema file %s to map your data does not exist ",
                    self.metadata_file,
                )
                sys.exit(1)
            with open(self.schema_file, "r") as fh:
                json_schema = json.load(fh)
            try:
                Draft202012Validator.check_schema(json_schema)
            except jsonschema.ValidationError:
                stderr.print(
                    "[red] Json schema does not fulfil Draft 202012 Validation"
                )
                sys.exit(1)
        elif self.destination_schema == "ENA":
            self.schema_file = os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                "schema",
                config_json.get_topic_data("json_schemas", "ena_schema"),
            )
        elif self.destination_schema == "GISAID":
            self.schema_file = os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                "schema",
                config_json.get_topic_data("json_schemas", "gisaid_schema"),
            )
        else:
            stderr.print("[red] Invalid option for mapping to schena")
            sys.exit(1)
        with open(self.schema_file, "r") as fh:
            self.mapped_to_schema = json.load(fh)

        self.ontology = {}
        for key, values in self.relecov_schema["properties"].items():
            if values["ontology"] == "0":
                continue
            self.ontology[values["ontology"]] = key
        self.output_folder = output_folder

    def maping_schemas_based_on_geontology(self):
        """Return a dictionnary with the properties of the mapped_to_schema as key and
        properties of Relecov Schema as value
        """
        mapped_dict = OrderedDict()
        for key, values in self.mapped_to_schema["properties"].items():
            if values["ontology"] == "0":
                continue
            try:
                mapped_dict[key] = self.ontology[values["ontology"]]
            except KeyError as e:
                log.error("Invalid ontology for %s", key)
                stderr.print(f"[red] Ontology value {e} not in relecov schema")
        return mapped_dict

    def mapping_json_data(self, mapping_schema_dict):
        """Convert phage plus data to the requested schema"""
        mapped_data = []

        for data in self.json_data:
            map_sample_dict = OrderedDict()
            for item, value in mapping_schema_dict.items():
                try:

                    data[value] = data[value].split(" [", 1)[0]

                    map_sample_dict[item] = data[value]
                except KeyError as e:
                    log.warning("Property %s not set in the source data", e)
            mapped_data.append(map_sample_dict)
        return mapped_data

    def additional_formating(self, mapped_json_data):
        """Update data like MD5 to split in two fields, one for R1 file and
        second for R2
        """
        if self.destination_schema == "ENA":
            for idx in range(len(self.json_data)):

                mapped_json_data[idx]["collector_name"] = "unknown"
                mapped_json_data[idx]["file_format"] = "FASTQ"
                mapped_json_data[idx]["study_type"] = "Whole Genome Sequencing"
                mapped_json_data[idx][
                    "study_title"
                ] = "RELECOV Spanish Network for genomics surveillance"
                mapped_json_data[idx]["study_abstract"] = mapped_json_data[idx][
                    "study_title"
                ]
                mapped_json_data[idx]["broker_name"] = "Instituto de Salud Carlos III"
                mapped_json_data[idx]["host health state"] = "not provided"
                mapped_json_data[idx]["center_name"] = self.json_data[idx][
                    "collecting_institution"
                ]
                mapped_json_data[idx]["library_name"] = self.json_data[idx][
                    "library_preparation_kit"
                ]
                mapped_json_data[idx]["sample_title"] = self.json_data[idx][
                    "sequencing_sample_id"
                ]
                mapped_json_data[idx]["sample_name"] = self.json_data[idx][
                    "sequencing_sample_id"
                ]
                mapped_json_data[idx]["host subject id"] = self.json_data[idx][
                    "sequencing_sample_id"
                ]
                mapped_json_data[idx]["study_alias"] = self.json_data[idx][
                    "schema_name"
                ]

                mapped_json_data[idx]["sample_description"] = (
                    self.json_data[idx]["anatomical_part"].split(" [", 1)[0]
                    + " "
                    + self.json_data[idx]["collection_method"].split(" [", 1)[0]
                )
                mapped_json_data[idx]["isolate"] = self.json_data[idx][
                    "isolate_sample_id"
                ]
                mapped_json_data[idx]["platform"] = self.json_data[idx][
                    "sequencing_instrument_platform"
                ].upper()

                mapped_json_data[idx]["authors"] = self.json_data[idx]["authors"]
                mapped_json_data[idx]["design_description"] = (
                    self.json_data[idx]["library_layout"].split(" [", 1)[0]
                    + " "
                    + self.json_data[idx]["library_preparation_kit"].split(" [", 1)[0]
                    + " "
                    + self.json_data[idx]["library_selection"].split(" [", 1)[0]
                    + " "
                    + self.json_data[idx]["library_source"].split(" [", 1)[0]
                    + "  "
                    + self.json_data[idx]["library_strategy"].split(" [", 1)[0]
                )
                mapped_json_data[idx]["insert_size"] = "0"
                mapped_json_data[idx]["address"] = self.json_data[idx][
                    "collecting_institution_address"
                ]
                "Adding all the 'not provided' fields that are not being captured"
                config_json = ConfigJson()
                fields = config_json.get_configuration("ENA_fields")[
                    "map_not_provided_fields"
                ]
                mapped_json_data[idx]["r1_fastq_filepath"] = (
                    self.json_data[idx]["r1_fastq_filepath"]
                    + self.json_data[idx]["sequence_file_R1_fastq"]
                )
                mapped_json_data[idx]["r2_fastq_filepath"] = (
                    self.json_data[idx]["r2_fastq_filepath"]
                    + self.json_data[idx]["sequence_file_R2_fastq"]
                )
                for i, j in enumerate(fields):

                    mapped_json_data[idx][j] = "not provided"

        return mapped_json_data

    def write_json_fo_file(self, mapped_json_data):
        """Write metadata to json file"""
        os.makedirs(self.output_folder, exist_ok=True)
        time = datetime.now().strftime("%Y_%m_%d_%H_%M")
        f_sub_name = os.path.basename(self.json_file).split(".")[0]
        file_name = (
            f_sub_name + "_" + time + "_" + self.destination_schema + "_mapped.json"
        )
        json_file = os.path.join(self.output_folder, file_name)
        with open(json_file, "w", encoding="utf-8") as fh:
            fh.write(
                json.dumps(
                    mapped_json_data, indent=4, sort_keys=True, ensure_ascii=False
                )
            )
        return True

    def map_to_data_to_new_schema(self):
        """Mapping the json data from relecov schema to the requested one"""
        mapping_schema_dict = self.maping_schemas_based_on_geontology()
        mapped_json_data = self.mapping_json_data(mapping_schema_dict)
        updated_json_data = self.additional_formating(mapped_json_data)
        self.write_json_fo_file(updated_json_data)
        return



#!/usr/bin/env python
from datetime import datetime
from itertools import islice
import copy
import logging
import glob
import json
import rich.console
import paramiko
import sys
import os
import shutil
import yaml
import openpyxl
import relecov_tools.utils
from relecov_tools.config_json import ConfigJson
from relecov_tools.rest_api import RestApi

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class SftpHandle:
    def __init__(
        self,
        user=None,
        passwd=None,
        conf_file=None,
        user_relecov=None,
        password_relecov=None,
    ):
        """Initializes the sftp object"""
        config_json = ConfigJson()
        self.allowed_sample_ext = config_json.get_configuration(
            "allowed_sample_extensions"
        )
        self.sftp_user = user
        self.sftp_passwd = passwd
        if conf_file is None:
            self.sftp_server = config_json.get_topic_data(
                "sftp_connection", "sftp_server"
            )
            self.sftp_port = config_json.get_topic_data("sftp_connection", "sftp_port")
            self.storage_local_folder = config_json.get_configuration(
                "storage_local_folder"
            )
            self.metadata_tmp_folder = config_json.get_configuration(
                "tmp_folder_for_metadata"
            )
            self.abort_if_md5_mismatch = (
                True
                if config_json.get_configuration("abort_if_md5_mismatch") == "True"
                else False
            )
        else:
            if not os.path.isfile(conf_file):
                log.error("Configuration file %s does not exists", conf_file)
                stderr.print(
                    "[red] Configuration file does not exist. " + conf_file + "!"
                )
                sys.exit(1)
            with open(conf_file, "r") as fh:
                config = yaml.load(fh, Loader=yaml.FullLoader)
            try:
                self.sftp_server = config["sftp_server"]
                self.sftp_port = config["sftp_port"]
                try:
                    self.storage_local_folder = config["storage_local_folder"]
                except KeyError:
                    self.storage_local_folder = config_json.get_configuration(
                        "storage_local_folder"
                    )
                try:
                    self.metadata_tmp_folder = config["tmp_metadata_folder"]
                except KeyError:
                    self.metadata_tmp_folder = config_json.get_configuration(
                        "tmp_folder_for_metadata"
                    )
                try:
                    self.abort_if_md5_mismatch = (
                        True if config["abort_if_md5_mismatch"] == "True" else False
                    )
                except KeyError:
                    self.abort_if_md5_mismatch = (
                        True
                        if config_json.get_configuration("abort_if_md5_mismatch")
                        == "True"
                        else False
                    )
                self.sftp_user = config["sftp_user"]
                self.sftp_passwd = config["sftp_passwd"]
                self.pp = config["allowed_sample_extensions"]
            except KeyError as e:
                log.error("Invalid configuration file %s", e)
                stderr.print("[red] Invalide configuration file {e} !")
                sys.exit(1)
        if self.sftp_user is None:
            self.sftp_user = relecov_tools.utils.prompt_text(msg="Enter the userid")
        if self.sftp_passwd is None:
            self.sftp_passwd = relecov_tools.utils.prompt_password(
                msg="Enter your password"
            )
        self.client = None
        external_servers = config_json.get_topic_data("external_url", "relecov")
        self.relecov_server = external_servers["server"]
        self.relecov_url = external_servers["url"]
        self.relecov_post_url = external_servers["sftp_info"]

    def open_connection(self):
        """Establish sftp connection"""
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.client.connect(
            hostname=self.sftp_server,
            port=self.sftp_port,
            username=self.sftp_user,
            password=self.sftp_passwd,
            allow_agent=False,
            look_for_keys=False,
        )
        try:
            self.client = self.client.open_sftp()
            return True
        except paramiko.SSHException as e:
            log.error("Invalid Username/Password for %s:", e)
            return False

    def close_connection(self):
        """Closes SFTP connection"""
        self.client.close()
        return True

    def list_folders(self, folder_name):
        """Creates a directories list from the given path"""
        directory_list = []
        try:
            content_list = self.client.listdir(folder_name)
        except FileNotFoundError as e:
            log.error("Invalid folder at remote sftp %s", e)
            return False

        for content in content_list:
            try:
                self.client.listdir(content)
            except FileNotFoundError:
                continue
            directory_list.append(content)

        return directory_list

    def get_file_list(self, folder_name):
        """Return a tuple with file name and directory path"""
        file_list = []
        content_list = self.client.listdir(folder_name)
        for content in content_list:
            try:
                sub_folder_files = self.client.listdir(folder_name + "/" + content)
                for sub_folder_file in sub_folder_files:
                    file_list.append(
                        folder_name + "/" + content + "/" + sub_folder_file
                    )
            except FileNotFoundError:
                file_list.append(folder_name + "/" + content)

        return file_list

    def get_files_from_sftp_folder(self, folder, files_list):
        """Create the subfolder with the present date and fetch all files from
        the sftp server
        """
        result_data = {"unable_to_fetch": [], "fetched_files": []}
        date = datetime.today().strftime("%Y%m%d")
        local_folder_path = os.path.join(self.storage_local_folder, folder, date)
        result_data["local_folder"] = local_folder_path
        os.makedirs(local_folder_path, exist_ok=True)
        log.info("created the folder to download files %s", local_folder_path)
        self.open_connection()

        for file_list in files_list:
            try:
                self.client.get(
                    file_list,
                    os.path.join(local_folder_path, os.path.basename(file_list)),
                )
            except FileNotFoundError as e:
                log.error("Unable to fetch file %s ", e)
                result_data["Unable_to_fetch"].append(file_list)
                continue
            result_data["fetched_files"].append(os.path.basename(file_list))
        return result_data

    def verify_md5_checksum(self, local_folder, file_list):
        """Get the md5 value from sftp match with the generated at local
        folder
        """
        required_retransmition = []
        successful_files = []
        # fetch the md5 file if exists
        sftp_md5 = relecov_tools.utils.get_md5_from_local_folder(local_folder)
        if len(sftp_md5) > 0:
            # check md5 checksum for eac file
            for f_name, values in sftp_md5.items():
                f_path_name = os.path.join(local_folder, f_name)
                # Checksum value is stored in the index 1
                if values[1] == relecov_tools.utils.calculate_md5(f_path_name):
                    log.info(
                        "Successful file download for %s in folder %s",
                        f_name,
                        local_folder,
                    )
                    successful_files.append(f_name)
                else:
                    required_retransmition.append(f_name)
                    log.error("%s requested file re-sending", f_name)

        if len(set(file_list)) != len(sftp_md5) * 2:
            # create the md5 file from the ones not upload to server
            req_create_md5 = [
                v
                for v in file_list
                if (v not in successful_files and not v.endswith("*.md5"))
            ]
            sftp_md5.update(
                relecov_tools.utils.create_md5_files(local_folder, req_create_md5)
            )
        return sftp_md5, required_retransmition

    def create_tmp_files_with_metadata_info(
        self, local_folder, file_list, md5_data, metadata_file
    ):
        """Copy metadata file from folder and create a file with the sample
        names
        """
        out_folder = self.metadata_tmp_folder
        os.makedirs(out_folder, exist_ok=True)
        prefix_file_name = "_".join(local_folder.split("/")[-2:])
        new_metadata_file = "metadata_lab_" + prefix_file_name + ".xlsx"
        sample_data_file = "samples_data_" + prefix_file_name + ".json"
        sample_data_path = os.path.join(out_folder, sample_data_file)
        try:
            shutil.copy(
                os.path.join(local_folder, metadata_file),
                os.path.join(out_folder, new_metadata_file),
            )
        except OSError as e:
            log.error("Unable to copy Metadata file %s", e)
            stderr.print("[red] Unable to copy Metadata file")
            return False
        data = copy.deepcopy(file_list)
        for s_name, values in file_list.items():
            for f_type, f_name in values.items():
                if not f_name.endswith(tuple(self.allowed_sample_ext)):
                    stderr.print("[red] " + f_name + " has a not valid extension")
                if "_R1_" in f_name:
                    data[s_name]["r1_fastq_filepath"] = md5_data[f_name][0]
                    data[s_name]["fastq_r1_md5"] = md5_data[f_name][1]
                elif "_R2_" in f_name:
                    data[s_name]["r2_fastq_filepath"] = md5_data[f_name][0]
                    data[s_name]["fastq_r2_md5"] = md5_data[f_name][1]
                else:
                    pass
        with open(sample_data_path, "w", encoding="utf-8") as fh:
            fh.write(json.dumps(data, indent=4, sort_keys=True, ensure_ascii=False))

        return True

    def create_main_folders(self, root_directory_list):
        """Create the main folder structure if not exists"""
        for folder in root_directory_list:
            full_folder = os.path.join(self.storage_local_folder, folder)
            os.makedirs(full_folder, exist_ok=True)
        return True

    def find_metadata_file(self, local_folder):
        """Find excel extension file which contains metatada"""
        reg_for_xlsx = os.path.join(local_folder, "*.xlsx")
        ex_files = glob.glob(reg_for_xlsx)
        if len(ex_files) == 0:
            log.error("Excel file for metadata does not exist on %s", local_folder)
            stderr.print("[red] Metadata file does not exist on " + local_folder)
            return False
        if len(ex_files) > 1:
            log.error("Too many Excel files on folder %s", local_folder)
            stderr.print("[red] Metadata file does not exist on " + local_folder)
            return False
        return ex_files[0]

    def get_sample_fastq_file_names(self, local_folder, meta_f_path):
        """ """
        if not os.path.isfile(meta_f_path):
            log.error("Metadata file does not exists on %s", local_folder)
            stderr.print("[red] METADATA_LAB.xlsx do not exist in" + local_folder)
            return False
        wb_file = openpyxl.load_workbook(meta_f_path, data_only=True)
        ws_metadata_lab = wb_file["METADATA_LAB"]
        sample_file_list = {}
        # find out the index for file names
        for col in ws_metadata_lab[4]:
            if "Sequence file R1 fastq" == col.value:
                index_fastq_r1 = col.column - 1
            elif "Sequence file R2 fastq" == col.value:
                index_fastq_r2 = col.column - 1
        for row in islice(ws_metadata_lab.values, 4, ws_metadata_lab.max_row):
            if row[2] is not None:
                try:
                    s_name = str(row[2])
                except ValueError as e:
                    stderr.print("[red] Unable to convert to string. ", e)
                if s_name not in sample_file_list:
                    sample_file_list[s_name] = {}
                else:
                    print("Found duplicated sample ", s_name)
                if row[index_fastq_r1] is not None:
                    sample_file_list[s_name]["sequence_file_R1_fastq"] = row[
                        index_fastq_r1
                    ]
                else:
                    log.error(
                        "Fastq_R1 not defined in Metadata file for sample %s", s_name
                    )
                    stderr.print("[red] No fastq R1 file for sample " + s_name)
                    return False
                if row[index_fastq_r2] is not None:
                    sample_file_list[s_name]["sequence_file_R2_fastq"] = row[
                        index_fastq_r2
                    ]
        return sample_file_list

    def validate_download_files(self, sample_file_list, local_folder):
        """Check if download sample files are the ones defined on metadata file"""
        if not sample_file_list:
            return False

        for sample, files in sample_file_list.items():
            for file in files.values():
                if not os.path.isfile(os.path.join(local_folder, file)):
                    log.error("File %s does not exist", file)
                    stderr.print("[red] File " + file + " does not exist")
                    return False
        return True

    def delete_remote_files(self, folder, files):
        """Delete files from remote server"""
        self.open_connection()
        for file in files:
            try:
                self.client.remove(file)
                log.info("%s Deleted from remote server", file)
            except FileNotFoundError:
                continue
        return

    def create_json_with_downloaded_samples(self, sample_file_list, folder):
        """From the download information prepare a json file"""
        sample_dict = {}
        for sample in sample_file_list:
            sample_dict[sample] = {"folder": folder}
        return json.dumps(sample_dict, indent=4, sort_keys=True, ensure_ascii=False)

    def send_info_to_server(self, json_data):
        rest_api = RestApi(self.database_server, self.database_url)
        error_counts = 0
        for item in json_data:
            result = rest_api.post_request(item, "", self.relecov_post_url)
            if "ERROR" in result:
                error_counts += 1
                log.error("Request was not accepted %s", result["ERROR"])
                stderr.print(
                    "[red] Error " + result["ERROR"] + " when sending request}"
                )
        if error_counts == 0:
            log.info("Request was not accepted %s", result["ERROR"])
            stderr.print("[green] Successful upload information to server")
        return

    def delete_local_folder(self, local_folder):
        """Delete download folder because files does not complain requisites"""
        shutil.rmtree(local_folder, ignore_errors=True)
        return True

    def download(self):
        try:
            os.makedirs(self.storage_local_folder, exist_ok=True)
        except OSError as e:
            log.error("You do not have permissions to create folder %s", e)
            sys.exit(1)
        os.chdir(self.storage_local_folder)
        if not self.open_connection():
            log.error("Unable to establish connection towards sftp server")
            stderr.print("[red] Unable to establish sftp connection")
            sys.exit(1)

        root_directory_list = self.list_folders(".")
        if not root_directory_list:
            log.error("There is no folders under root directory")
            sys.exit(1)
        folders_to_download = {}
        # create_main_folders(root_directory_list)
        for folder in root_directory_list:
            list_files = self.get_file_list(folder)
            if len(list_files) > 0:
                folders_to_download[folder] = list_files
        if len(folders_to_download) == 0:
            log.info("Exiting download. There is no files on sftp to dowload")
            self.close_connection()
            sys.exit(0)

        for folder, files in folders_to_download.items():
            log.info("Processing folder %s", folder)
            stderr.print("Processing folder " + folder)
            # get the files in each folder
            result_data = self.get_files_from_sftp_folder(folder, files)
            md5_files, req_retransmition = self.verify_md5_checksum(
                result_data["local_folder"], result_data["fetched_files"]
            )
            # retrasmision of files in folder
            if len(req_retransmition) > 0:
                restransmition_data = self.get_files_from_sftp_folder(
                    folder, req_retransmition
                )
                md5_ret_files, corrupted = self.verify_md5_checksum(
                    result_data["local_folder"], restransmition_data["fetched_files"]
                )
                md5_files.update(md5_ret_files)
                if len(corrupted) > 0 and self.abort_if_md5_mismatch:
                    log.error("Stopping because of corrupted files %s", corrupted)
                    stderr.print(
                        f"[red] Stop processing folder {folder} because of corrupted files {corrupted}"
                    )
                    continue
            meta_file = self.find_metadata_file(result_data["local_folder"])
            sample_file_list = self.get_sample_fastq_file_names(
                result_data["local_folder"], meta_file
            )
            if self.validate_download_files(
                sample_file_list, result_data["local_folder"]
            ):
                self.create_tmp_files_with_metadata_info(
                    result_data["local_folder"], sample_file_list, md5_files, meta_file
                )
                # Collect data to send the request to relecov_platform
                json_sample_data = self.create_json_with_downloaded_samples(
                    sample_file_list, folder
                )
                for record in json_sample_data:
                    pass
            else:
                log.info("Deleting local folder %s", result_data["local_folder"])
                self.delete_local_folder(result_data["local_folder"])

        self.close_connection()
        return



# from distutils.errors import LibError
import logging

# from re import template

# from pyparsing import col
import xml.etree.ElementTree as ET
import rich.console
import json


import pandas as pd


import sys
import os

import ftplib
import relecov_tools.utils
from relecov_tools.config_json import ConfigJson

from ena_upload.ena_upload import extract_targets

# from ena_upload.ena_upload import submit_data
from ena_upload.ena_upload import run_construct
from ena_upload.ena_upload import construct_submission
from ena_upload.ena_upload import send_schemas
from ena_upload.ena_upload import process_receipt
from ena_upload.ena_upload import update_table

# from ena_upload.ena_upload import make_update
# from ena_upload.ena_upload import process_receipt

# from ena_upload.ena_upload import save_update
import site

pd.options.mode.chained_assignment = None

template_path = os.path.join(site.getsitepackages()[0], "ena_upload", "templates")
template_path = os.path.join(os.getcwd(), "relecov_tools/templates")

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class EnaUpload:
    def __init__(
        self,
        user=None,
        passwd=None,
        center=None,
        source_json=None,
        dev=None,
        customized_project=None,
        action=None,
        output_path=None,
    ):
        if user is None:
            self.user = relecov_tools.utils.prompt_text(
                msg="Enter your username defined in ENA"
            )
        else:
            self.user = user
        if passwd is None:
            self.passwd = relecov_tools.utils.prompt_password(
                msg="Enter your password to ENA"
            )
        else:
            self.passwd = passwd
        if center is None:
            self.center = relecov_tools.utils.prompt_text(msg="Enter your center name")
        else:
            self.center = center
        if source_json is None:
            self.source_json_file = relecov_tools.utils.prompt_path(
                msg="Select the ENA json file to upload"
            )
        else:
            self.source_json_file = source_json
        if dev is None:
            self.dev = relecov_tools.utils.prompt_yn_question(
                msg="Do you want to test upload data?"
            )
        else:
            self.dev = dev

        if customized_project is None:
            self.customized_project = None
        else:
            self.customized_project = customized_project
        if action is None:
            self.action = relecov_tools.utils.prompt_selection(
                msg="Select the action to upload to ENA",
                choices=["add", "modify", "cancel", "release"],
            )
        else:
            self.action = action.upper()
        if output_path is None:
            self.output_path = relecov_tools.utils.prompt_path(
                msg="Select the folder to store the xml files"
            )
        else:
            self.output_path = output_path

        if not os.path.isfile(self.source_json_file):
            log.error("json data file %s does not exist ", self.source_json_file)
            stderr.print(f"[red]json data file {self.source_json_file} does not exist")
            sys.exit(1)
        with open(self.source_json_file, "r") as fh:
            self.json_data = json.loads(fh.read())

    def convert_input_json_to_ena(self):
        """Split the input ena json, in samples and runs json"""
        pass

    def create_dataframe(self, dataframe_name, fields_config, df_schemas):
        df_list = []
        config_json = ConfigJson()
        for index in range(len(dataframe_name)):

            fields = config_json.get_configuration("ENA_fields")[fields_config[index]]
            dataframe_name[index] = df_schemas[fields]
            df_list.append(dataframe_name[index])

        return df_list

    def rename_cols_df(self, dataframe_name, dataframe, col_name_og, col_name_final):
        col_dict = {}

        for index in range(len(col_name_og)):
            col_dict[col_name_og[index]] = col_name_final[index]

        dataframe_name = dataframe.rename(columns=col_dict)

        return dataframe_name

    def create_structure_to_ena(self):
        """Convert json to dataframe required by ena-upload-cli package"""

        squema = self.source_json_file
        fh_squema = open(squema)
        squema_json = json.load(fh_squema)
        fh_squema.close()
        config_json = ConfigJson()

        df_schemas = pd.DataFrame(squema_json)

        dataframe_name_list = ["df_study", "df_samples", "df_run", "df_experiments"]
        dataframe_fileds_config_list = [
            "df_study_fields",
            "df_samples_fields",
            "df_run_fields",
            "df_experiment_fields",
        ]
        # Creating the dataframes df_Study, df_samples, df_run and df_experiment
        df_list = self.create_dataframe(
            dataframe_name_list, dataframe_fileds_config_list, df_schemas
        )

        df_study = df_list[0]
        df_samples = df_list[1]
        df_run = df_list[2]
        df_experiments = df_list[3]

        # df_schema
        """
        collecting_institution collection_date     collector_name  ...                                        study_title    study_type taxon_id
0      Hospital Clínic de Barcelona      2021-05-10  Inmaculada Casas   ...  RELECOV Spanish Network for genomics surveillance  Surveillance  2697049
1      Hospital Clínic de Barcelona      2021-05-07   Inmaculada Casas  ...  RELECOV Spanish Network for genomics surveillance  Surveillance  2697049
        """
        # Parsing the dataframes specifically according to ENA requirements #
        # df_study
        df_study["alias"] = df_study["study_alias"]
        df_study["title"] = df_study["study_title"]
        df_study.insert(3, "status", self.action)

        """
        # df_study
        alias                                              title    study_type status                                     study_abstract
0   RELECOV  RELECOV Spanish Network for genomics surveillance  Surveillance    ADD  RELECOV Spanish Network for genomics surveillance
1   RELECOV  RELECOV Spanish Network for genomics surveillance  Surveillance    ADD  RELECOV Spanish Network for genomics surveillance
        """

        # df_samples
        df_samples = self.rename_cols_df(
            "df_samples",
            df_samples,
            config_json.get_configuration("ENA_fields")["rename_sample_list_og"],
            config_json.get_configuration("ENA_fields")["rename_sample_list_final"],
        )
        df_samples.insert(3, "status", self.action)
        checklist = config_json.get_configuration("ENA_fields")["checklist"]
        df_samples.insert(4, "ENA_CHECKLIST", checklist)

        """
        # df_samples
        alias      title taxon_id host health state  ...                                  scientific_name     collector name           collecting institution    isolate
0   212164375  212164375  2697049                    ...  Severe acute respiratory syndrome coronavirus 2  Inmaculada Casas      Hospital Clínic de Barcelona  212164375
1   212163777  212163777  2697049                    ...  Severe acute respiratory syndrome coronavirus 2   Inmaculada Casas     Hospital Clínic de Barcelona  212163777
2   212153091  212153091  2697049                    ...  Severe acute respiratory syndrome coronavirus 2   Inmaculada Casas     Hospital Clínic de Barcelona  212153091
        """
        # df_run
        df_run.insert(1, "sequence_file_R1_fastq", "None")
        df_run.insert(2, "sequence_file_R2_fastq", "None")
        for i in range(len(df_schemas)):
            df_run.loc[i, "sequence_file_R1_fastq"] = df_schemas.loc[
                i, "sequence_file_R1_fastq"
            ]

            df_run.loc[i, "sequence_file_R2_fastq"] = df_schemas.loc[
                i, "sequence_file_R2_fastq"
            ]
        df_run.insert(3, "status", self.action)

        df_run = df_run.rename(columns={"sequence_file_R1_md5": "file_checksum"})

        for i in range(len(df_run)):
            df_run.loc[i, "alias"] = (
                str(df_run.loc[i, "sequence_file_R1_fastq"])
                + "_"
                + str(df_run.loc[i, "sequence_file_R2_fastq"])
            )
            df_run.loc[i, "experiment_alias"] = (
                str(df_run.loc[i, "sequence_file_R1_fastq"])
                + "_"
                + str(df_run.loc[i, "sequence_file_R2_fastq"])
            )

        df_run.insert(5, "file_name", df_run["sequence_file_R1_fastq"])
        df_run2 = df_run.copy()
        df_run2["file_name"] = df_run["sequence_file_R2_fastq"]
        df_run2["file_checksum"] = df_run["sequence_file_R2_md5"]
        df_run_final = pd.concat([df_run, df_run2])
        df_run_final.reset_index()

        """
        # df_run
        experiment_alias      sequence_file_R1_fastq      sequence_file_R2_fastq  ... file_type                     file_checksum                      fastq_r2_md5
0        214821_S12  214821_S12_R1_001.fastq.gz  214821_S12_R2_001.fastq.gz  ...     fastq  372ca8b10a8eeb7a04107634baf340ab  7f5081eec1b64b171402b66f37fe640d
1        214821_S12  214822_S13_R1_001.fastq.gz  214822_S13_R2_001.fastq.gz  ...     fastq  b268d7be80e80455bec0807b5961d23c  a15be39feaf73eec9b2c026717878bba
2        214821_S12   214823_S1_R1_001.fastq.gz   214823_S1_R2_001.fastq.gz  ...     fastq  c16bdbfc03c354496fcfb2c107e3cbf6  4d9b80b977a75bf7e2a4282ca910d94a
        """
        # df_experiments

        df_experiments["instrument_model"] = df_experiments[
            "instrument_model"
        ].str.lower()
        df_experiments["sample_alias"] = df_samples["alias"]
        df_experiments["title"] = df_study["study_abstract"]
        df_experiments["study_alias"] = df_study["alias"]
        df_experiments.insert(3, "status", self.action)

        for i in range(len(df_experiments)):
            df_experiments.loc[i, "alias"] = (
                str(df_run.loc[i, "sequence_file_R1_fastq"])
                + "_"
                + str(df_run.loc[i, "sequence_file_R2_fastq"])
            )

        """
        # df_experiments example
        experiment_alias                                              title study_alias  ... library_layout instrument_model                                              alias
0        214821_S12  RELECOV Spanish Network for genomics surveillance     RELECOV  ...         PAIRED   Illumina MiSeq  214821_S12_R1_001.fastq.gz_214821_S12_R2_001.f...
1        214821_S12  RELECOV Spanish Network for genomics surveillance     RELECOV  ...         PAIRED   Illumina MiSeq  214822_S13_R1_001.fastq.gz_214822_S13_R2_001.f...
2        214821_S12  RELECOV Spanish Network for genomics surveillance     RELECOV  ...         PAIRED   Illumina MiSeq  214823_S1_R1_001.fastq.gz_214823_S1_R2_001.fas...

        """

        ena_config = config_json.get_configuration("ENA_fields")["ENA_configuration"]
        schema_dataframe = {}
        schema_dataframe["sample"] = df_samples
        schema_dataframe["run"] = df_run_final
        schema_dataframe["experiment"] = df_experiments
        schema_targets = extract_targets(self.action, schema_dataframe)

        if ena_config["study_id"] is not None:
            schema_dataframe["study"] = df_study

        if (
            self.action == "ADD"
            or self.action == "add"
            or self.action == "MODIFY"
            or self.action == "modify"
        ):
            file_paths = {}
            file_paths_r2 = {}
            for path in df_schemas["r1_fastq_filepath"]:
                file_paths[os.path.basename(path)] = os.path.abspath(path)

            for path in df_schemas["r2_fastq_filepath"]:
                file_paths_r2[os.path.basename(path)] = os.path.abspath(path)

            file_paths.update(file_paths_r2)

            # Submit data to webin ftp server. It should only upload fastq files in case the action is ADD.
            # When the action is MODIFY rthe fastq are already submitted

            if self.action == "ADD" or self.action == "add":
                session = ftplib.FTP("webin2.ebi.ac.uk", self.user, self.passwd)

                for filename, path in file_paths.items():

                    print("Uploading path " + path + " and filename: " + filename)

                    try:
                        file = open(path, "rb")  # file to send
                        g = session.storbinary(f"STOR {filename}", file)
                        print(g)  # send the file
                        file.close()  # close file and FTP
                    except BaseException as err:

                        print(f"ERROR: {err}")
                        # print("ERROR: If your connection times out at this stage, it propably is because of a firewall that is in place. FTP is used in passive mode and connection will be opened to one of the ports: 40000 and 50000.")

                g2 = session.quit()
                print(g2)

            """
            THE ENA_UPLOAD_CLI METHOD DOES NOT WORK (below)
            chec = submit_data(file_paths, self.passwd, self.user)
            print(chec)
            When ADD/MODIFY:
            requires source XMLs for 'run', 'experiment', 'sample', 'experiment'
            schema_xmls record XMLs for all these schema and following 'submission'
            """
            #
            tool = config_json.get_configuration("ENA_fields")["tool"]

            schema_xmls = run_construct(
                template_path, schema_targets, self.center, checklist, tool
            )

            submission_xml = construct_submission(
                template_path, self.action, schema_xmls, self.center, checklist, tool
            )
            schema_xmls["submission"] = submission_xml

            tree = ET.parse(schema_xmls["run"])
            root = tree.getroot()

            for files in root.iter("FILE"):
                if "R2" in files.attrib["filename"]:

                    H = df_run_final.loc[
                        df_run_final["sequence_file_R2_fastq"]
                        == files.attrib["filename"]
                    ].values[0][8]
                    files.set("checksum", H)

            tree.write(schema_xmls["run"])

            if self.dev:
                url = "https://wwwdev.ebi.ac.uk/ena/submit/drop-box/submit/?auth=ENA"
            else:
                url = "https://www.ebi.ac.uk/ena/submit/drop-box/submit/?auth=ENA"

            print(f"\nSubmitting XMLs to ENA server: {url}")

            receipt = send_schemas(schema_xmls, url, self.user, self.passwd).text
            if not os.path.exists(self.output_path):
                os.mkdir(self.output_path)
            receipt_dir = os.path.join(self.output_path, "receipt.xml")
            print(f"Printing receipt to {receipt_dir}")

            with open(f"{receipt_dir}", "w") as fw:
                fw.write(receipt)
            try:
                schema_update = process_receipt(receipt.encode("utf-8"), self.action)
            except ValueError:
                log.error("There was an ERROR during submission:")
                sys.exit(receipt)

            if self.action in ["ADD", "MODIFY"] or self.action in ["add", "modify"]:
                schema_dataframe = update_table(
                    schema_dataframe, schema_targets, schema_update
                )

    def upload(self):
        """Create the required files and upload to ENA"""
        self.convert_input_json_to_ena()
        self.create_structure_to_ena()



#!/usr/bin/env python
"""
Common utility function used for relecov_tools package.
"""
import os
import glob
import hashlib
import logging
import questionary
import json
import openpyxl
import yaml
from itertools import islice
from Bio import SeqIO
from rich.console import Console

log = logging.getLogger(__name__)


def file_exists(file_to_check):
    """
    Input:
        file_to_check   # file name to check if exists
    Return:
        True if exists
    """
    if os.path.isfile(file_to_check):
        return True
    return False


def get_files_match_condition(condition):
    """find all path names that matches with the condition"""
    return glob.glob(condition)


def read_json_file(j_file):
    """Read json file."""
    with open(j_file, "r") as fh:
        try:
            data = json.load(fh)
        except (UnicodeDecodeError, ValueError):
            raise

    return data


def read_excel_file(f_name, sheet_name, heading_row, empty_value=True):
    """Read the input excel file and give the information in a list
    of dictionaries
    """
    wb_file = openpyxl.load_workbook(f_name, data_only=True)
    ws_metadata_lab = wb_file[sheet_name]
    heading = [i.value.strip() for i in ws_metadata_lab[heading_row] if i.value]
    ws_data = []
    for row in islice(ws_metadata_lab.values, heading_row, ws_metadata_lab.max_row):
        l_row = list(row)
        data_row = {}
        # Ignore the empty rows
        # guessing that row 1 and 2 with no data are empty rows
        if l_row[0] is None and l_row[1] is None:
            continue
        for idx in range(0, len(heading)):
            if l_row[idx] is None:
                if empty_value:
                    data_row[heading[idx]] = ""
                else:
                    data_row[heading[idx]] = "Not Provided"
            else:
                data_row[heading[idx]] = l_row[idx]
        ws_data.append(data_row)

    return ws_data


def read_csv_file_return_dict(file_name, sep, key_position=None):
    """Read csv or tsv file, according to separator, and return a dictionary
    where the main key is the first column, if key position is None otherwise
    the index value of the kwy position is used as key
    """

    try:
        with open(file_name, "r") as fh:
            lines = fh.readlines()
    except FileNotFoundError:
        raise
    heading = lines[0].strip().split(sep)
    if len(heading) == 1:
        return {"ERROR": "not valid format"}
    file_data = {}
    for line in lines[1:]:
        line_s = line.strip().split(sep)
        if key_position is None:
            file_data[line_s[0]] = {}
            for idx in range(1, len(heading)):
                file_data[line_s[0]][heading[idx]] = line_s[idx]
        else:
            file_data[line_s[key_position]] = {}
            for idx in range(len(heading)):
                if idx == key_position:
                    continue
                file_data[line_s[key_position]][heading[idx]] = line_s[idx]

    return file_data


def read_fasta_return_SeqIO_instance(file_name):
    """Read fasta and return SeqIO instance"""
    try:
        return SeqIO.read(file_name, "fasta")
    except FileNotFoundError:
        raise


def read_yml_file(file_name):
    """Read yml file"""
    with open(file_name, "r") as fh:
        try:
            return yaml.safe_load(fh)
        except yaml.YAMLError:
            raise


def get_md5_from_local_folder(local_folder):
    """Fetch the md5 values for each file in the file list"""
    md5_results = {}
    reg_for_md5 = os.path.join(local_folder, "*.md5")
    # reg_for_non_md5 = os.path.join(local_folder, "*[!.md5]")
    md5_files = glob.glob(reg_for_md5)
    if len(md5_files) > 0:
        for md5_file in md5_files:
            file_path_name, f_ext = os.path.splitext(md5_file)
            if not file_exists(file_path_name):
                log.error("Found md5 file but not %s", file_path_name)
                continue
            file_name = os.path.basename(file_path_name)
            fh = open(md5_file, "r")
            md5_results[file_name] = [local_folder, fh.read()]
            fh.close()
    return md5_results


def calculate_md5(file_name):
    """Calculate the md5 value for the file name"""
    return hashlib.md5(open(file_name, "rb").read()).hexdigest()


def write_md5_file(file_name, md5_value):
    """Write md5 to file"""
    with open(file_name, "w") as fh:
        fh.write(md5_value)
    return


def create_md5_files(local_folder, file_list):
    """Create the md5 files and return their value"""
    md5_results = {}
    for file_name in file_list:
        md5_results[file_name] = [
            local_folder,
            calculate_md5(os.path.join(local_folder, file_name)),
        ]
        md5_file_name = file_name + ".md5"
        write_md5_file(
            os.path.join(local_folder, md5_file_name), md5_results[file_name][1]
        )
    return md5_results


def save_local_md5(file_name, md5_value):
    """Save the MD5 value"""
    with open(file_name, "w") as fh:
        fh.write(md5_value)
    return True


def write_json_fo_file(data, file_name):
    """Write metadata to json file"""
    with open(file_name, "w", encoding="utf-8") as fh:
        fh.write(json.dumps(data, indent=4, sort_keys=True, ensure_ascii=False))
    return True


def write_to_excel_file(data, f_name, sheet_name, post_process=None):
    book = openpyxl.Workbook()
    sheet = book.active

    for row in data:
        sheet.append(row)
    # adding one column with row number
    if "insert_cols" in post_process:
        sheet.insert_cols(post_process["insert_cols"])
        sheet["A1"] = "Campo"
        counter = 1
        for i in range(len(data) - 1):
            idx = "A" + str(counter + 1)
            sheet[idx] = counter
            counter += 1
    # adding 3 empty rows
    if "insert_rows" in post_process:
        for x in range(post_process["insert_rows"]):
            sheet.insert_rows(1)
        sheet.title = sheet_name
    book.save(f_name)
    return


def rich_force_colors():
    """
    Check if any environment variables are set to force Rich to use coloured output
    """
    if (
        os.getenv("GITHUB_ACTIONS")
        or os.getenv("FORCE_COLOR")
        or os.getenv("PY_COLORS")
    ):
        return True
    return None


stderr = Console(
    stderr=True, style="dim", highlight=False, force_terminal=rich_force_colors()
)


def prompt_text(msg):
    source = questionary.text(msg).unsafe_ask()
    return source


def prompt_password(msg):
    source = questionary.password(msg).unsafe_ask()
    return source


def prompt_tmp_dir_path():
    stderr.print("Temporal directory destination to execute service")
    source = questionary.path("Source path").unsafe_ask()
    return source


def prompt_selection(msg, choices):
    selection = questionary.select(msg, choices=choices).unsafe_ask()
    return selection


def prompt_path(msg):
    source = questionary.path(msg).unsafe_ask()
    return source


def prompt_yn_question(msg):
    confirmation = questionary.confirm(msg).unsafe_ask()
    return confirmation


def prompt_skip_folder_creation():
    stderr.print("Do you want to skip folder creation? (Y/N)")
    confirmation = questionary.confirm("Skip?", default=False).unsafe_ask()
    return confirmation



#!/usr/bin/env python
import logging

# import re

# from rich.prompt import Confirm
import click
import rich.console
import rich.logging
import rich.traceback

import relecov_tools.utils
import relecov_tools.read_lab_metadata
import relecov_tools.sftp_handle
import relecov_tools.ena_upload
import relecov_tools.json_validation
import relecov_tools.map_schema
import relecov_tools.feed_database
import relecov_tools.read_bioinfo_metadata
import relecov_tools.long_table_parse
import relecov_tools.metadata_homogeneizer
import relecov_tools.gisaid_upload

log = logging.getLogger()

# Set up rich stderr console
stderr = rich.console.Console(
    stderr=True, force_terminal=relecov_tools.utils.rich_force_colors()
)


def run_relecov_tools():

    # Set up the rich traceback
    rich.traceback.install(console=stderr, width=200, word_wrap=True, extra_lines=1)

    # Print nf-core header
    # stderr.print("\n[green]{},--.[grey39]/[green],-.".format(" " * 42), highlight=False)
    stderr.print(
        "[blue]                ___   ___       ___  ___  ___                           ",
        highlight=False,
    )
    stderr.print(
        "[blue]   \    |-[grey39]-|  [blue] |   \ |    |    |    |    |   | \      /  ",
        highlight=False,
    )
    stderr.print(
        "[blue]    \   \  [grey39]/ [blue]  |__ / |__  |    |___ |    |   |  \    /   ",
        highlight=False,
    )
    stderr.print(
        "[blue]    /  [grey39] / [blue] \   |  \  |    |    |    |    |   |   \  /    ",
        highlight=False,
    )
    stderr.print(
        "[blue]   /   [grey39] |-[blue]-|   |   \ |___ |___ |___ |___ |___|    \/     ",
        highlight=False,
    )

    # stderr.print("[green]                                          `._,._,'\n", highlight=False)
    __version__ = "0.0.5"
    stderr.print(
        "\n" "[grey39]    RELECOV-tools version {}".format(__version__), highlight=False
    )

    # Lanch the click cli
    relecov_tools_cli()


# Customise the order of subcommands for --help
class CustomHelpOrder(click.Group):
    def __init__(self, *args, **kwargs):
        self.help_priorities = {}
        super(CustomHelpOrder, self).__init__(*args, **kwargs)

    def get_help(self, ctx):
        self.list_commands = self.list_commands_for_help
        return super(CustomHelpOrder, self).get_help(ctx)

    def list_commands_for_help(self, ctx):
        """reorder the list of commands when listing the help"""
        commands = super(CustomHelpOrder, self).list_commands(ctx)
        return (
            c[1]
            for c in sorted(
                (self.help_priorities.get(command, 1000), command)
                for command in commands
            )
        )

    def command(self, *args, **kwargs):
        """Behaves the same as `click.Group.command()` except capture
        a priority for listing command names in help.
        """
        help_priority = kwargs.pop("help_priority", 1000)
        help_priorities = self.help_priorities

        def decorator(f):
            cmd = super(CustomHelpOrder, self).command(*args, **kwargs)(f)
            help_priorities[cmd.name] = help_priority
            return cmd

        return decorator


@click.group(cls=CustomHelpOrder)
@click.version_option(relecov_tools.__version__)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Print verbose output to the console.",
)
@click.option(
    "-l", "--log-file", help="Save a verbose log to a file.", metavar="<filename>"
)
def relecov_tools_cli(verbose, log_file):

    # Set the base logger to output DEBUG
    log.setLevel(logging.DEBUG)

    # Set up logs to a file if we asked for one
    if log_file:
        log_fh = logging.FileHandler(log_file, encoding="utf-8")
        log_fh.setLevel(logging.DEBUG)
        log_fh.setFormatter(
            logging.Formatter(
                "[%(asctime)s] %(name)-20s [%(levelname)-7s]  %(message)s"
            )
        )
        log.addHandler(log_fh)


# sftp
@relecov_tools_cli.command(help_priority=2)
@click.option("-u", "--user", help="User name for login to sftp server")
@click.option("-p", "--password", help="password for the user to login")
@click.option("-r_u", "--user_relecov", help="User name for updating data to relecov")
@click.option("-p_r", "--password_relecov", help="password for relecov user")
@click.option(
    "-f",
    "--conf_file",
    help="Configuration file (no params file)",
)
def download(user, password, conf_file, user_relecov, password_relecov):
    """Download files located in sftp server."""
    sftp_connection = relecov_tools.sftp_handle.SftpHandle(
        user, password, conf_file, user_relecov, password_relecov
    )
    sftp_connection.download()


# metadata
@relecov_tools_cli.command(help_priority=3)
@click.option(
    "-m",
    "--metadata_file",
    type=click.Path(),
    help="file containing metadata",
)
@click.option(
    "-s",
    "--sample_list_file",
    type=click.Path(),
    help="Json with the additional metadata to add to the received user metadata",
)
@click.option(
    "-o", "--metadata-out", type=click.Path(), help="Path to save output  metadata file"
)
def read_lab_metadata(metadata_file, sample_list_file, metadata_out):
    """
    Create the json compliant to the relecov schema from the Metadata file.
    """
    new_metadata = relecov_tools.read_lab_metadata.RelecovMetadata(
        metadata_file, sample_list_file, metadata_out
    )
    relecov_json = new_metadata.create_metadata_json()
    return relecov_json


# validation
@relecov_tools_cli.command(help_priority=4)
@click.option("-j", "--json_file", help="Json file to validate")
@click.option("-s", "--json_schema", help="Json schema")
@click.option(
    "-m",
    "--metadata",
    type=click.Path(),
    help="Origin file containing metadata",
)
@click.option("-o", "--out_folder", help="Path to save validate json file")
def validate(json_file, json_schema, metadata, out_folder):
    """Validate json file against schema."""
    validation = relecov_tools.json_validation.SchemaValidation(
        json_file, json_schema, metadata, out_folder
    )
    validation.validate()


# mapping to ENA schema
@relecov_tools_cli.command(help_priority=5)
@click.option("-p", "--origin_schema", help="File with the origin (relecov) schema")
@click.option("-j", "--json_data", help="File with the json data to convert")
@click.option(
    "-d",
    "--destination_schema",
    type=click.Choice(["ENA", "GISAID", "other"], case_sensitive=True),
    help="schema to be mapped",
)
@click.option("-f", "--schema_file", help="file with the custom schema")
@click.option("-o", "--output", help="File name and path to store the mapped json")
def map(origin_schema, json_data, destination_schema, schema_file, output):
    """Convert data between phage plus schema to ENA, GISAID, or any other schema"""
    new_schema = relecov_tools.map_schema.MappingSchema(
        origin_schema, json_data, destination_schema, schema_file, output
    )
    new_schema.map_to_data_to_new_schema()


# upload to ENA
@relecov_tools_cli.command(help_priority=6)
@click.option("-u", "--user", help="user name for login to ena")
@click.option("-p", "--password", help="password for the user to login")
@click.option("-c", "--center", help="center name")
@click.option("-e", "--ena_json", help="where the validated json is")
@click.option("-s", "--study", help="study/project name to include in xml files")
@click.option(
    "-a",
    "--action",
    type=click.Choice(["add", "modify", "cancel", "release"], case_sensitive=False),
    help="select one of the available options",
)
@click.option("--dev", is_flag=True, default=False)
@click.option("-o", "--output_path", help="output folder for the xml generated files")
def upload_to_ena(user, password, center, ena_json, dev, study, action, output_path):
    """parsed data to create xml files to upload to ena"""

    upload_ena = relecov_tools.ena_upload.EnaUpload(
        user, password, center, ena_json, dev, study, action, output_path
    )
    upload_ena.upload()


# upload to GISAID
@relecov_tools_cli.command(help_priority=7)
@click.option("-u", "--user", help="user name for login")
@click.option("-p", "--password", help="password for the user to login")
@click.option("-c", "--client_id", help="client-ID provided by clisupport@gisaid.org")
@click.option("-t", "--token", help="path to athentication token")
@click.option("-e", "--gisaid_json", help="path to validated json mapped to GISAID")
@click.option(
    "-i",
    "--input_path",
    help="path to fastas folder or multifasta file",
)
@click.option("-o", "--output_path", help="output folder for log")
@click.option(
    "-f",
    "--frameshift",
    type=click.Choice(["catch_all", "catch_none", "catch_novel"], case_sensitive=False),
    help="frameshift notification",
)
@click.option(
    "-x",
    "--proxy_config",
    help="introduce your proxy credentials as: username:password@proxy:port",
    required=False,
)
@click.option(
    "--single",
    is_flag=True,
    default=False,
    help="input is a folder with several fasta files. Default: False",
)
@click.option(
    "--gzip",
    is_flag=True,
    default=False,
    help="input fasta is gziped. Default: False",
)
def upload_to_gisaid(
    user,
    password,
    client_id,
    token,
    gisaid_json,
    input_path,
    output_path,
    frameshift,
    proxy_config,
    single,
    gzip,
):
    """parsed data to create files to upload to gisaid"""
    upload_gisaid = relecov_tools.gisaid_upload.GisaidUpload(
        user,
        password,
        client_id,
        token,
        gisaid_json,
        input_path,
        output_path,
        frameshift,
        proxy_config,
        single,
        gzip,
    )
    upload_gisaid.gisaid_upload()


# launch
@relecov_tools_cli.command(help_priority=8)
@click.option("-u", "--user", help="user name for connecting to the server")
def launch(user):
    """launch viralrecon in hpc"""
    pass


# update_db
@relecov_tools_cli.command(help_priority=9)
@click.option("-j", "--json", help="data in json format")
@click.option(
    "-t",
    "--type",
    type=click.Choice(["sample", "bioinfodata", "variantdata"]),
    multiple=False,
    help="Select the type of information to upload to database",
)
@click.option(
    "-d",
    "databaseServer",
    type=click.Choice(
        [
            "iskylims",
            "relecov",
        ]
    ),
    multiple=False,
    help="name of the server which information is defined in config file",
)
@click.option("-u", "--user", help="user name for login")
@click.option("-p", "--password", help="password for the user to login")
def update_db(user, password, json, type, databaseServer):
    """feed database with json"""
    feed_database = relecov_tools.feed_database.FeedDatabase(
        user, password, json, type, databaseServer
    )
    feed_database.store_data()


# read metadata bioinformatics
@relecov_tools_cli.command(help_priority=10)
@click.option(
    "-j",
    "--json_file",
    type=click.Path(),
    help="json file containing lab metadata",
)
@click.option("-i", "--input_folder", type=click.Path(), help="Path to input files")
@click.option("-o", "--out_dir", type=click.Path(), help="Path to save output file")
def read_bioinfo_metadata(json_file, input_folder, out_dir):
    """
    Create the json compliant  from the Bioinfo Metadata.
    """
    new_bioinfo_metadata = relecov_tools.read_bioinfo_metadata.BioinfoMetadata(
        json_file, input_folder, out_dir
    )

    new_bioinfo_metadata.create_bioinfo_file()


# read metadata bioinformatics
@relecov_tools_cli.command(help_priority=11)
@click.option(
    "-l",
    "--longtable_file",
    type=click.Path(),
    help="file containing variant long table ",
)
@click.option("-o", "--output", type=click.Path(), help="Path to save json output")
def long_table_parse(longtable_file, output):
    new_json_parse = relecov_tools.long_table_parse.LongTableParse(
        longtable_file, output
    )
    """Create Json file from long table"""
    new_json_parse.parsing_csv()


# read metadata bioinformatics
@relecov_tools_cli.command(help_priority=12)
@click.option(
    "-i",
    "--institution",
    type=click.Choice(["isciii", "hugtip", "hunsc-iter"], case_sensitive=False),
    help="select one of the available institution options",
)
@click.option(
    "-d",
    "--directory",
    type=click.Path(),
    help="Folder where are located the additional files",
)
@click.option("-o", "--output", type=click.Path(), help="Path to save json output")
def metadata_homogeneizer(institution, directory, output):
    """Parse institution metadata lab to the one used in relecov"""
    new_parse = relecov_tools.metadata_homogeneizer.MetadataHomogeneizer(
        institution, directory, output
    )
    new_parse.converting_metadata()


if __name__ == "__main__":
    run_relecov_tools()



#!/usr/bin/env python
import logging
import rich.console
import jsonschema
from jsonschema import Draft202012Validator
import sys
import os
import openpyxl

import relecov_tools.utils
from relecov_tools.config_json import ConfigJson

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class SchemaValidation:
    def __init__(
        self, json_data_file=None, json_schema_file=None, metadata=None, out_folder=None
    ):
        """Validate json file against the schema"""

        if json_schema_file is None:
            config_json = ConfigJson()
            schema_name = config_json.get_topic_data("json_schemas", "relecov_schema")
            json_schema_file = os.path.join(
                os.path.dirname(os.path.realpath(__file__)), "schema", schema_name
            )

        self.json_schema = relecov_tools.utils.read_json_file(json_schema_file)

        if json_data_file is None:
            json_data_file = relecov_tools.utils.prompt_path(
                msg="Select the json file to validate"
            )

        if out_folder is None:
            self.out_folder = relecov_tools.utils.prompt_path(
                msg="Select the folder where to save excel with invalid data"
            )
        else:
            self.out_folder = out_folder

        # Read and check json to validate file
        if not os.path.isfile(json_data_file):
            stderr.print("[red] Json file does not exists")
            sys.exit(1)

        stderr.print("[blue] Reading the json file")
        self.json_data = relecov_tools.utils.read_json_file(json_data_file)

        self.metadata = metadata

    def validate_schema(self):
        """Validate json schema against draft"""
        try:
            Draft202012Validator.check_schema(self.json_schema)
        except jsonschema.ValidationError:
            stderr.print("[red] Json schema does not fulfill Draft 202012 Validation")
            sys.exit(1)

    def validate_instances(self):
        """Validate data instances against a validated json schema"""

        # create validator
        validator = Draft202012Validator(self.json_schema)

        validated_json_data = []
        invalid_json = []
        errors = {}
        stderr.print("[blue] Start processing the json file")
        for item_row in self.json_data:
            # validate(instance=item_row, schema=json_schema)
            if validator.is_valid(item_row):
                validated_json_data.append(item_row)
            else:
                # Count error types
                for error in validator.iter_errors(item_row):
                    if error.message in errors.keys():
                        errors[error.message] += 1
                    else:
                        errors[error.message] = 1

                # log.error("Invalid sample data %s", error.message)
                # append row with errors
                invalid_json.append(item_row)

        # Summarize errors
        stderr.print("[blue] --------------------")
        stderr.print("[blue] VALIDATION SUMMARY")
        stderr.print("[blue] --------------------")
        for error_type in errors.keys():
            stderr.print(
                "[red]"
                + str(errors[error_type])
                + " samples failed validation:\n"
                + error_type
            )
            stderr.print("[red] --------------------")

        return invalid_json

    def create_invalid_metadata(self, invalid_json, metadata, out_folder):
        """Create a new sub excel file having only the samples that were invalid.
        Samples name are checking the Sequencing sample id which are in
        column B (index 1).
        The rows that match the value collected from json file on tag
        collecting_lab_sample_id are removed from excel
        """
        if len(invalid_json) == 0:
            stderr.print(
                "[green] Sucessful validation, no invalid file will be written!!"
            )
        else:
            log.error("Some of the samples in json metadata were not validated")
            stderr.print("[red] Some of the Samples are not validate")
            if metadata is None:
                metadata = relecov_tools.utils.prompt_path(
                    msg="Select the metadata file to select those samples not validated."
                )
            if not os.path.isfile(metadata):
                log.error("Metadata file %s does not exist", metadata)
                stderr.print(
                    "[red] Unable to create excel file for invalid samples. Metadata file ",
                    metadata,
                    " does not exist",
                )
                sys.exit(1)

            sample_list = []

            stderr.print("[red] Start preparation of invalid samples")

            for row in invalid_json:
                sample_list.append(str(row["collecting_lab_sample_id"]))

            wb = openpyxl.load_workbook(metadata)
            ws_sheet = wb["METADATA_LAB"]
            row_to_del = []

            for row in ws_sheet.iter_rows(min_row=5, max_row=ws_sheet.max_row):
                # if not data on row 1 and 2 assume that no more data are in file
                # then start deleting rows
                if not row[2].value and not row[1].value:
                    break
                if str(row[2].value) not in sample_list:
                    row_to_del.append(row[0].row)

            stderr.print("[red] Collected rows to create the excel file")
            if len(row_to_del) > 0:
                row_to_del.sort(reverse=True)
                for idx in row_to_del:
                    try:
                        ws_sheet.delete_rows(idx)
                    except TypeError as e:
                        log.error(
                            "Unable to delete row %s from metadata file because of",
                            idx,
                            e,
                        )
                        stderr.print("f[red] Unable to delete row {idx} becuase of {e}")
                        sys.exit(1)

            os.makedirs(out_folder, exist_ok=True)
            new_name = "invalid_" + os.path.basename(metadata)
            m_file = os.path.join(out_folder, new_name)
            stderr.print("[red] Saving excel file with the invalid samples")
            wb.save(m_file)
        return

    def validate(self):
        """Write invalid samples from metadata to excel"""

        self.validate_schema()
        invalid_json = self.validate_instances()
        self.create_invalid_metadata(invalid_json, self.metadata, self.out_folder)

        # Enviar los errores por correo
        # logging.handlers.SMTPHandler(mailhost=("smtp.gmail.com", 465), fromaddr=correo_isciii, toaddrs=correo_usuario, subject="Validation errors", credentials=(usurario,contraseña), secure=None, timeout=1.0)



#!/usr/bin/env python
import json
import os


# pass test
class ConfigJson:
    def __init__(
        self,
        json_file=os.path.join(os.path.dirname(__file__), "conf", "configuration.json"),
    ):
        fh = open(json_file)
        self.json_data = json.load(fh)
        fh.close()
        self.topic_config = list(self.json_data.keys())

    def get_configuration(self, topic):
        """Obtain the topic configuration from json data"""
        if topic in self.topic_config:
            return self.json_data[topic]
        return None

    def get_topic_data(self, topic, found):
        """Obtain from topic any forward items from json data"""
        if found in self.json_data[topic]:
            return self.json_data[topic][found]
        else:
            for key, value in self.json_data[topic].items():
                if isinstance(value, dict):
                    if found in self.json_data[topic][key]:
                        return self.json_data[topic][key][found]
            return None



#!/usr/bin/env python
import json
import logging
import rich.console
import os
import sys
import re
import relecov_tools.utils
from relecov_tools.config_json import ConfigJson
import relecov_tools.json_schema

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)
#


class RelecovMetadata:
    def __init__(self, metadata_file=None, sample_list_file=None, output_folder=None):
        if metadata_file is None:
            self.metadata_file = relecov_tools.utils.prompt_path(
                msg="Select the excel file which contains metadata"
            )
        else:
            self.metadata_file = metadata_file

        if not os.path.exists(self.metadata_file):
            log.error("Metadata file %s does not exist ", self.metadata_file)
            stderr.print(
                "[red] Metadata file " + self.metadata_file + " does not exist"
            )
            sys.exit(1)

        if sample_list_file is None:
            self.sample_list_file = relecov_tools.utils.prompt_path(
                msg="Select the file which contains the sample information"
            )
        else:
            self.sample_list_file = sample_list_file

        if not os.path.exists(self.sample_list_file):
            log.error(
                "Sample information file %s does not exist ", self.sample_list_file
            )
            stderr.print(
                "[red] Sample information " + self.sample_list_file + " does not exist"
            )
            sys.exit(1)

        if output_folder is None:
            self.output_folder = relecov_tools.utils.prompt_path(
                msg="Select the output folder"
            )
        else:
            self.output_folder = output_folder

        config_json = ConfigJson()
        relecov_schema = config_json.get_topic_data("json_schemas", "relecov_schema")
        relecov_sch_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "schema", relecov_schema
        )
        self.configuration = config_json

        with open(relecov_sch_path, "r") as fh:
            self.relecov_sch_json = json.load(fh)
        self.label_prop_dict = {}

        for prop, values in self.relecov_sch_json["properties"].items():
            try:
                self.label_prop_dict[values["label"]] = prop
            except KeyError:
                log.warning("Property %s does not have 'label' attribute", prop)
                stderr.print(
                    "[orange] Property " + prop + " does not have 'label' attribute"
                )
                continue

        self.json_req_files = config_json.get_topic_data(
            "lab_metadata", "lab_metadata_req_json"
        )
        self.schema_name = self.relecov_sch_json["title"]
        self.schema_version = self.relecov_sch_json["version"]

    def adding_fixed_fields(self, m_data):
        """Include fixed data that are always the same for each samples"""
        p_data = self.configuration.get_topic_data("lab_metadata", "fixed_fields")
        for idx in range(len(m_data)):
            for key, value in p_data.items():
                m_data[idx][key] = value
            m_data[idx]["schema_name"] = self.schema_name
            m_data[idx]["schema_version"] = self.schema_version
        return m_data

    def adding_copy_from_other_field(self, m_data):
        """Add in a new field the information that is already set in another
        field.
        """
        p_data = self.configuration.get_topic_data(
            "lab_metadata", "required_copy_from_other_field"
        )
        for idx in range(len(m_data)):
            for key, value in p_data.items():
                m_data[idx][key] = m_data[idx][value]
        return m_data

    def adding_post_processing(self, m_data):
        """Add the fields that requires to set based on the existing value
        in other field
        """
        p_data = self.configuration.get_topic_data(
            "lab_metadata", "required_post_processing"
        )
        for idx in range(len(m_data)):
            for key, p_values in p_data.items():
                value = m_data[idx][key]
                if value in p_values:
                    p_field, p_set = p_values[value].split("::")
                    m_data[idx][p_field] = p_set
                else:
                    # Check if key p_values should match only part of the value
                    for reg_key, reg_value in p_values.items():
                        if reg_key in value:
                            p_field, p_set = reg_value.split("::")
                            m_data[idx][p_field] = p_set

        return m_data

    def adding_ontology_to_enum(self, m_data):
        """Read the schema to get the properties enum and for those fields
        which has an enum property value then replace the value for the one it
        is defined in the schema
        """
        enum_dict = {}
        for prop, values in self.relecov_sch_json["properties"].items():
            if "enum" in values:
                enum_dict[prop] = {}
                for enum in values["enum"]:
                    go_match = re.search(r"(.+) \[\w+:.*", enum)
                    if go_match:
                        enum_dict[prop][go_match.group(1)] = enum
                    else:
                        enum_dict[prop][enum] = enum

        for idx in range(len(m_data)):
            for key, e_values in enum_dict.items():
                if key in m_data[idx]:
                    if m_data[idx][key] in e_values:
                        m_data[idx][key] = e_values[m_data[idx][key]]
                    else:
                        continue

        return m_data

    def process_from_json(self, m_data, json_fields):
        """ """
        map_field = json_fields["map_field"]

        json_data = json_fields["j_data"]
        if isinstance(json_data, dict):
            for idx in range(len(m_data)):
                m_data[idx].update(json_data[m_data[idx][map_field]])
        elif isinstance(json_data, list):
            # to avoid searching for data for each row, for the first time searchs
            # it is stored temporary.
            tmp_data = {}
            for idx in range(len(m_data)):
                if m_data[idx][map_field] not in tmp_data:
                    for item in json_data:
                        if m_data[idx][map_field] == item[map_field]:
                            if json_fields["adding_fields"] == "__all__":
                                m_data[idx].update(item)
                                tmp_data[m_data[idx][map_field]] = item
                            else:
                                tmp_data[m_data[idx][map_field]] = {}
                                for field in json_fields["adding_fields"]:
                                    m_data[idx][field] = item[field]
                                    tmp_data[m_data[idx][map_field]][field] = item[
                                        field
                                    ]
                            break
                else:
                    m_data[idx].update(tmp_data[m_data[idx][map_field]])
        return m_data

    def adding_fields(self, metadata):
        """Add information that requires to handle json files to include in
        the  fields"""

        for key, values in self.json_req_files.items():
            stderr.print(f"[blue] Processing {key}")
            f_path = os.path.join(
                os.path.dirname(os.path.realpath(__file__)), "conf", values["file"]
            )
            values["j_data"] = relecov_tools.utils.read_json_file(f_path)
            metadata = self.process_from_json(metadata, values)
            stderr.print(f"[green] Processed {key}")

        # Because sample data file is comming in an input parameter it cannot
        # be inside the configuration json file.
        # Include Sample informatin data from sample json file
        stderr.print("[blue] Processing sample data file")
        s_json = {}
        s_json["map_field"] = "sequencing_sample_id"
        s_json["adding_field"] = "__all__"
        s_json["j_data"] = relecov_tools.utils.read_json_file(self.sample_list_file)
        metadata = self.process_from_json(metadata, s_json)
        stderr.print("[green] Processed sample data file.")
        return metadata

    def read_configuration_json_files(self):
        """Read json files defined in configuration lab_metadata_req_json
        property
        """
        c_files = {}
        for item, value in self.json_files.items():
            f_path = os.path.join(
                os.path.dirname(os.path.realpath(__file__)), "conf", value
            )
            c_files[item] = relecov_tools.utils.read_json_file(f_path)
        return c_files

    def read_metadata_file(self):
        """Read the input metadata file from row 4, changes the metadata heading
        with their property name values defined in schema.
        Convert the date colunms value to the dd/mm/yyyy format.
        Return list of dict with data, and errors
        """
        heading_row_number = 4
        ws_metadata_lab = relecov_tools.utils.read_excel_file(
            self.metadata_file, "METADATA_LAB", heading_row_number, False
        )
        metadata_values = []
        errors = {}
        row_number = heading_row_number
        for row in ws_metadata_lab:
            row_number += 1
            property_row = {}
            try:
                sample_number = row["Sample ID given for sequencing"]
            except KeyError:
                log.error(
                    "Sample ID given for sequencing not found in row  %s", row_number
                )
                stderr.print(
                    f"[red] Sample ID given for sequencing not found in row {row_number}"
                )
                continue
            for key in row.keys():
                # skip the first column of the Metadata lab file
                if "Campo" in key:
                    continue
                if "date" in key.lower():
                    if row[key] is not None:
                        try:
                            row[key] = row[key].strftime("%Y-%m-%d")
                        except AttributeError:
                            # check if date is in string format
                            str_date = re.search(r"(\d{4}-\d{2}-\d{2}).*", row[key])
                            if str_date:
                                row[key] = str_date.group(1)
                            else:
                                if sample_number not in errors:
                                    errors[sample_number] = {}
                                errors[sample_number][key] = "Invalid date format"
                                log.error(
                                    "Invalid date format in sample %s", row_number
                                )
                                stderr.print(
                                    f"[red] Invalid date format in sample {sample_number},  {key}"
                                )
                elif "sample id" in key.lower():
                    if isinstance(row[key], float) or isinstance(row[key], int):
                        row[key] = str(int(row[key]))

                else:
                    if isinstance(row[key], float) or isinstance(row[key], int):
                        row[key] = str(row[key])
                try:
                    property_row[self.label_prop_dict[key]] = row[key]
                except KeyError as e:
                    log.error("Error when mapping the label %s", e)
                    stderr.print(f"[red] Error when mapping the label {str(e)}")
                    continue

            metadata_values.append(property_row)
        return metadata_values, errors

    def create_metadata_json(self):
        # stderr.print("[blue] Reading configuration settings")
        # conf_json_data = self.read_configuration_json_files()
        stderr.print("[blue] Reading Lab Metadata Excel File")
        valid_metadata_rows, errors = self.read_metadata_file()
        if len(errors) > 0:
            stderr.print("[red] Stopped executing because the errors found")
            sys.exit(1)
        # Continue by adding extra information
        stderr.print("[blue] Including additional information")

        extended_metadata = self.adding_fields(valid_metadata_rows)
        stderr.print("[blue] Including post processing information")
        extended_metadata = self.adding_post_processing(extended_metadata)
        extended_metadata = self.adding_copy_from_other_field(extended_metadata)
        extended_metadata = self.adding_fixed_fields(extended_metadata)
        completed_metadata = self.adding_ontology_to_enum(extended_metadata)
        file_name = (
            "processed_"
            + os.path.splitext(os.path.basename(self.metadata_file))[0]
            + ".json"
        )
        stderr.print("[blue] Writting output json file")
        os.makedirs(self.output_folder, exist_ok=True)
        file_path = os.path.join(self.output_folder, file_name)
        relecov_tools.utils.write_json_fo_file(completed_metadata, file_path)
        return True



#!/usr/bin/env python

from setuptools import setup, find_packages

version = "0.0.5"

with open("README.md") as f:
    readme = f.read()

with open("requirements.txt") as f:
    required = f.read().splitlines()

setup(
    name="relecov_tools",
    version=version,
    description="Tools for managing and resolution of buisciii services.",
    long_description=readme,
    long_description_content_type="text/markdown",
    keywords=[
        "buisciii",
        "bioinformatics",
        "pipeline",
        "sequencing",
        "NGS",
        "next generation sequencing",
    ],
    author="Sara Monzon",
    author_email="smonzon@isciii.es",
    url="https://github.com/BU-ISCIII/relecov-tools",
    license="GNU GENERAL PUBLIC LICENSE v.3",
    entry_points={
        "console_scripts": ["relecov-tools=relecov_tools.__main__:run_relecov_tools"]
    },
    install_requires=required,
    packages=find_packages(exclude=("docs")),
    include_package_data=True,
    zip_safe=False,
)



"""
=============================================================
HEADER
=============================================================
INSTITUTION: BU-ISCIII
AUTHOR: Guillermo J. Gorines Cordero
MAIL: guillermo.gorines@urjc.es
VERSION: 0
CREATED: 7-3-2022
REVISED: 7-3-2022
REVISED BY: guillermo.gorines@urjc.es
DESCRIPTION:

    Includes the Email, and its associated methods.

REQUIREMENTS:
    -Python

TO DO:


================================================================
END_OF_HEADER
================================================================
"""

# Imports
import smtplib

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


class Email:
    def __init__(self, receiver, sender, password, subject):
        self.receiver = receiver
        self.sender = sender
        self.password = password
        self.subject = subject
        self.text = ""
        self.html = False

    def write_message(self, text):
        self.message = text
        return

    def generate_HTML(self):
        pass
        return

    def send_message(self):
        msg = MIMEMultipart("alternative")
        msg["To"] = self.receiver
        msg["From"] = self.sender
        msg["Subject"] = self.subject

        text_part = MIMEText(self.text, "plain")
        msg.attach(text_part)

        if self.html:
            html_part = MIMEText(self.html, "html")
            msg.attach(html_part)

        # open server, send email, close email
        server = smtplib.SMTP("localhost")
        server.sendmail(self.sender, self.receiver, msg.as_string())
        server.quit()



import json
import re
import logging
import sys
import rich
import os.path

from pathlib import Path
from datetime import datetime

import relecov_tools
from relecov_tools.config_json import ConfigJson


# from relecov_tools.rest_api import RestApi

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)
#


class LongTableParse:
    """
    - parse_a_list_of_dictionaries() : returns generated_JSON
    - saving_file(generated_JSON)
    - parsing_csv() : It manages all this proccess:
       - calling first to parse_a_list_of_dictionaries() and then calling to saving_file()

    """

    def __init__(self, file_path=None, output_directory=None):
        if file_path is None:
            self.file_path = relecov_tools.utils.prompt_path(
                msg="Select the csv file which contains variant long table information"
            )
        else:
            self.file_path = file_path

        if not os.path.exists(self.file_path):
            log.error("Variant long table file %s does not exist ", self.file_path)
            stderr.print(
                f"[red] Variant long table file {self.file_path}  does not exist"
            )
            sys.exit(1)

        if not self.file_path.endswith(".csv"):
            log.error("Variant long table file %s is not a csv file ", self.file_path)
            stderr.print(
                f"[red] Variant long table file  {self.file_path}  must be a csv file"
            )
            sys.exit(1)

        if output_directory is None:
            self.output_directory = relecov_tools.utils.prompt_path(
                msg="Select the output folder"
            )
        else:
            self.output_directory = output_directory
        Path(self.output_directory).mkdir(parents=True, exist_ok=True)

        config_json = ConfigJson()
        self.long_table_heading = config_json.get_configuration("long_table_heading")

    def validate_file(self, heading):
        """Check if long table file has all mandatory fields defined in
        configuration file
        """
        for field in self.long_table_heading:
            if field not in heading:
                log.error("Incorrect format file. %s is missing", field)
                stderr.print(f"[red]Incorrect Format. {field} is missing in file")
                sys.exit(1)
        return True

    def parse_file(self):
        """This function generates a json file from the csv file entered by
        the user (long_table.csv).
        Validate the file by checking the header line
        """

        with open(self.file_path) as fh:
            lines = fh.readlines()

        self.validate_file(lines[0].strip().split(","))
        stderr.print("[green]Successful checking heading fields")
        stderr.print("[blue]Parsing the input file")
        heading_index = {}
        headings_from_csv = lines[0].strip().split(",")
        for heading in self.long_table_heading:
            heading_index[heading] = headings_from_csv.index(heading)

        samp_dict = {}
        for line in lines[1:]:
            line_s = line.strip().split(",")

            sample = line_s[heading_index["SAMPLE"]]
            if sample not in samp_dict:
                samp_dict[sample] = []
            variant_dict = {}
            variant_dict["Chromosome"] = line_s[heading_index["CHROM"]]

            variant_dict["Variant"] = {
                "pos": line_s[heading_index["POS"]],
                "alt": line_s[heading_index["ALT"]],
                "ref": line_s[heading_index["REF"]],
            }

            variant_dict["Filter"] = line_s[heading_index["FILTER"]]

            variant_dict["VariantInSample"] = {
                "dp": line_s[heading_index["DP"]],
                "ref_dp": line_s[heading_index["REF_DP"]],
                "alt_dp": line_s[heading_index["ALT_DP"]],
                "af": line_s[heading_index["AF"]],
            }

            variant_dict["Effect"] = line_s[heading_index["EFFECT"]]
            variant_dict["VariantAnnotation"] = {
                "hgvs_c": line_s[heading_index["HGVS_C"]],
                "hgvs_p": line_s[heading_index["HGVS_P"]],
                "hgvs_p_1_letter": line_s[heading_index["HGVS_P_1LETTER"]],
            }

            if re.search("&", line_s[heading_index["GENE"]]):
                # Example
                # 215184,NC_045512.2,27886,AAACGAACATGAAATT,A,PASS,1789,1756,1552,0.87,ORF7b&ORF8,gene_fusion,n.27887_27901delAACGAACATGAAATT,.,.,ivar,B.1.1.318
                # This only occurs (for now) as gene fusion, so we just duplicate lines with same values
                genes = re.split("&", line_s[heading_index["GENE"]])
                for gene in genes:
                    variant_dict_copy = variant_dict.copy()
                    variant_dict_copy["Gene"] = gene
                    samp_dict[sample].append(variant_dict_copy)
            else:
                variant_dict["Gene"] = line_s[heading_index["GENE"]]
                samp_dict[sample].append(variant_dict)

        stderr.print("[green]Successful parsing data")
        return samp_dict

    def convert_to_json(self, samp_dict):
        j_list = []
        # Grab date from filename
        result_regex = re.search(
            "variants_long_table_(.*).csv", os.path.basename(self.file_path)
        )
        for key, values in samp_dict.items():
            j_dict = {"sample_name": key, "analysis_date": result_regex.group(1)}
            j_dict["variants"] = values
            j_list.append(j_dict)
        return j_list

    def save_to_file(self, j_list):
        """Transform the parsed data into a jsonf file, naming as
        "long_table_" + "current date" + ".json"
        """
        stderr.print("[blue]Saving parsed data to file")
        date_now = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
        file_name = "long_table_" + date_now + ".json"
        file_path = os.path.join(self.output_directory, file_name)

        with open(file_path, "w") as fh:
            fh.write(json.dumps(j_list, indent=4))
        return

    def parsing_csv(self):
        """
        function called when using the relecov-tools long-table-parse function.
        """
        stderr.print("[blue]Starting reading the input file")
        parsed_data = self.parse_file()
        j_list = self.convert_to_json(parsed_data)
        self.save_to_file(j_list)
        stderr.print("[green]Process completed")
